//////////////////////////////////////////////////////////////////////////////
//
//                  ELEKTOR RFID READER for MIFARE and ISO14443-A
//
//                      Copyright (c) 2006 Gerhard H. Schalk
//         
//////////////////////////////////////////////////////////////////////////////
#include <REG936.H> 
#include <stdio.h>
#include <stdarg.h>
#include "Timer1.h"
#include "lcd.h" 

static bit displayPresent;

//------------------------------------------------------------------------
// Module internal function: busy_lcd
//------------------------------------------------------------------------
// Description:
// Reads the Busy Flag (BF) of the HD44780 LCD Display Controller and 
// returns either the BF Flag is cleared or an timeout occurred.
// The timeout typically occurs when there is no display is connected.
// or an timeout is occurred (e.g. no display connected).
// is active.
//------------------------------------------------------------------------
char busy_lcd(void)
{   
    data bit busy;
    unsigned int timeout = 1000;

    busy =1;
    while ((busy == 1)) 
    {
        D7=1; D6=1; D5=1; D4=1;  // Data Port is input (D7..busy flag)
        RS=0; RW=1;
        EN=1; busy=D7; EN=0;     // read 2. half of the data byte
        EN=1; EN=0;              // read 2. half of the data byte
    
        timeout--;
        if(timeout == 0)
            return -1;
    }
    return 0;              
 
}
  
//------------------------------------------------------------------------
// Function: blank_lcd
//------------------------------------------------------------------------
// Description:
// Clears the entire LCD display and sets the cursor to home position
//------------------------------------------------------------------------
void blank_lcd(void)          
{ 
    if (displayPresent == TRUE)
    {
        // CLEAR DISPLAY
        busy_lcd ();
        RS=0; RW=0;
        D7=0; D6=0; D5=0; D4=0; EN=1; EN=0;   // High-Byte
        D7=0; D6=0; D5=0; D4=1; EN=1; EN=0;   // Low-Byte
        
        // RETURN HOME
        busy_lcd ();
        RS=0; RW=0;
        D7=0; D6=0; D5=0; D4=0; EN=1; EN=0;   // High-Byte
        D7=0; D6=0; D5=1; D4=0; EN=1; EN=0;   // Low-Byte
    }
}

//------------------------------------------------------------------------
// Module internal function: putchar_lcd
//------------------------------------------------------------------------
// Description:
// The character c is written to the LCD display. 
// This function behaves like a normal putchar function.
//------------------------------------------------------------------------
// Parameters:
// IN  row:     LCD display row     (1 or 2)
// IN  column:  LCD display column  (1 to DISPLAY_MAX_COLUMNS)
// IN  c:       Character to display on the LCD
//------------------------------------------------------------------------
void putchar_lcd(char row, char column, char c)
{ 
    data char address;

    if (displayPresent == TRUE)
    {
        if ( (row > 0) && (row < (DISPLAY_MAX_RAWS +1 )))
        {
            if ( (column > 0) && (column < (DISPLAY_MAX_COLUMNS+1)) )
            { 
                 address = column - 1 + (row - 1) * 64;
                  
                 // DD RAM ADDRESS SET
                 busy_lcd();
                 RS=0; RW=0;
                 D7 = 1;              // Addr. H-Byte:
                 D6 = address & 64;     
                 D5 = address & 32;
                 D4 = address & 16;
                 EN=1; EN=0;          
                
                 D7 = address & 8;    // Addr. L-Byte:
                 D6 = address & 4;
                 D5 = address & 2;
                 D4 = address & 1;
                 EN=1; EN=0;          
                
                 // DD RAM DATA WRITE
                 busy_lcd(); 
                 RS=1; RW=0;
                 D7 = c & 128;   // Data H-Byte
                 D6 = c & 64;
                 D5 = c & 32;
                 D4 = c & 16;
                 EN=1; EN=0;
                
                 D7 = c & 8;    // Data L-Byte
                 D6 = c & 4;
                 D5 = c & 2;
                 D4 = c & 1;
                 EN=1; EN=0;
            }
        }
    }
}

//------------------------------------------------------------------------
// Function: init_lcd
//------------------------------------------------------------------------
// Description:
// Initialize the LCD-Controller HD44780
// 4-bit data bus; 2 Rows; 5x7 Dot
//------------------------------------------------------------------------
void init_lcd(void)
{
  char retValue;

  Timer1_Delay_ms(50);      // Wait 50ms
  D7=1; D6=1; D5=1; D4=1;
  RS=0; RW=0; EN=0;
  Timer1_Delay_ms(10);      // Wait 10ms

  // FUNC. SET: interface is 8 bit long  
  D7=0; D6=0; D5=1; D4=1; EN=1; EN=0; 
  Timer1_Delay_ms(10);      // Wait 10ms

  // FUNC. SET: interface is 8 bit long
  D7=0; D6=0; D5=1; D4=1; EN=1; EN=0; 
  Timer1_Delay_ms(10);      // Wait 10ms

    // FUNC. SET: interface is 8 bit long
  D7=0; D6=0; D5=1; D4=1; EN=1; EN=0; 
  Timer1_Delay_ms(10);      // Wait 10ms
  
  // FUNC.SET:interface is 4 bit long
  D7=0; D6=0; D5=1; D4=0; EN=1; EN=0;  
  Timer1_Delay_ms(10);      // Wait 10ms

  // FUNC.SET:interface is 4 bit long                                 
  D7=0; D6=0; D5=1; D4=0; EN=1; EN=0; // H-Byte: 4-bit Mode
  D7=1; D6=0; D5=0; D4=0; EN=1; EN=0; // L-Byte: 2 dislay lines; 5x7 Dot 

  // DISPLAY ON
  retValue = busy_lcd ();

  // Check if Display is present
  if(retValue == 0)
  {
       displayPresent = TRUE;
  }
  else
  {
       displayPresent = FALSE;
       return;
  }

  RS=0; RW=0;
  D7=0; D6=0; D5=0; D4=0; EN=1; EN=0; // H-Byte:    
  D7=1; D6=1; D5=0; D4=0; EN=1; EN=0; // L-Byte: Display ON, Cursor off

  // DISPLAY CLEAR
  retValue = busy_lcd ();
  RS=0; RW=0;
  D7=0; D6=0; D5=0; D4=0; EN=1; EN=0; // H-Byte:
  D7=0; D6=0; D5=0; D4=1; EN=1; EN=0; // L-Byte: clear display

  // ENTRY MODE SET
  busy_lcd ();
  RS=0; RW=0;
  D7=0; D6=0; D5=0; D4=0; EN=1; EN=0; // H-Byte:   
  D7=0; D6=1; D5=0; D4=0; EN=1; EN=0; // L-Byte: decrement, no shift 
}



//------------------------------------------------------------------------
// Module internal function: print_lcd
//------------------------------------------------------------------------
// Description:
// The printf_lcd function writes a ASCII string to the LCD display. 
//------------------------------------------------------------------------
// Parameters:
// IN  row:     LCD display row     (1 or 2)
// IN  column:  LCD display column  (1 to 40)
// IN  str:     ASCII string
//------------------------------------------------------------------------
void print_lcd(char row, char column, char *str)
{
    char i;
    
    if (displayPresent == TRUE)
    {
        for ( i = column; (i <= DISPLAY_MAX_COLUMNS+1) &&  *str; i++)
        {
            putchar_lcd(row,i,*str);
            str++;
        }
    }
}

//------------------------------------------------------------------------
// Function: printf_lcd
//------------------------------------------------------------------------
// Description:
//------------------------------------------------------------------------
// Funktion: void print_lcd(char row, char column, char *str)
// The printf_lcd function formats a series of strings and numeric 
// values and builds a string to write to the LCD display using the 
// vsprintf function.
// This function behaves like a normal printf function.
//------------------------------------------------------------------------
// Parameters:
// IN  row:     LCD display row     (1 or 2)
// IN  column:  LCD display column  (1 to 40)
// IN  *fmt:    format string 
//------------------------------------------------------------------------
void printf_lcd(char row, char column, const char *fmt,...)
{
   if (displayPresent == TRUE)
   {
       char lcdstring[DISPLAY_MAX_COLUMNS+1];
       unsigned int j=0;
       va_list arg_ptr;	
       va_start(arg_ptr,fmt); 
       j += vsprintf(lcdstring + j, fmt,arg_ptr);
       print_lcd(row, column, lcdstring);
       va_end(arg_ptr);
   }
}