//////////////////////////////////////////////////////////////////////////////
//
//                  ELEKTOR RFID READER for MIFARE and ISO14443-A
//
//                      Copyright (c) 2006 Gerhard H. Schalk
//         
//////////////////////////////////////////////////////////////////////////////
#include "RC522_Mifare.h"
#include "RC522_ErrCode.h"
#include "Mifare_Util.h"
#include "UART.h"
#include "LCD.h"
#include "EEPROM.h"
#include "string.h"
#include "LPC_Hardware.h"           // Output Pin definition
#include "Mifare_Access_Control.h"


// String variables for LC-Display user defined messages
unsigned char xdata abLCD_CardPolling_Line1[17];
unsigned char xdata abLCD_CardPolling_Line2[17];    

unsigned char xdata abLCD_Auth_Card_Line1[17];       
unsigned char xdata abLCD_Auth_Card_Line2[17];      

unsigned char xdata abLCD_Unauth_Card_Line1[17];     
unsigned char xdata abLCD_Unauth_Card_Line2[17];     


//------------------------------------------------------------------------
// Function: MIFARE_AccessControl
//------------------------------------------------------------------------
// Description: Mifare Access Control Demo:
// With the help of the PC windows application "Mifare Magic" it is 
// possible to store a list of 10 UID Card numbers in the LPC EEPROM.
// The MIFARE_AccessControl function polls for new Cards in the field and 
// simply compares the card UID of the current active card with the UID list.
// A card with a valid UID activates the open collector output.
// The configuration dialog of Mifare Magic also supports user defined 
// LC-display messages.
// MifareMagic 
//------------------------------------------------------------------------
void MIFARE_AccessControl(void)
{
        unsigned char abUID[8], bSAK;
        unsigned char baATQ[2], bUIDLen;
        unsigned char bAccessStatus;
        unsigned int  iEEAddr;
        unsigned char i,j;
        unsigned char baEE_UID_List_Entry[8];
        short status;

        
        bAccessStatus = NOT_ALLOWED;
        iEEAddr =  EEAddrUIDData;

        Rc522RFReset(5);  //RF - Reset 

        // Activate Card
        status = ActivateCard(ISO14443_3_REQA,baATQ,abUID,&bUIDLen,&bSAK);
        if(status !=  STATUS_SUCCESS)
        {
            OpenerOutput = 0;	// Clear (Door) Opener Output pin

            // Write to LCD Display Line 1            
            if(ReadEEPROM(EEAddrLCD_CardPolling_Line1) == 0xFF)
            {   
                // Default Text
                printf_lcd(1,1,"  ACCESS CTRL   "); // Write to LCD Display 
            }
            else
            {
                // EEPROM Configuration String
                printf_lcd(1,1,"%s",abLCD_CardPolling_Line1); // Write to LCD Display 
            }

            // Write to LCD Display Line 2   
            if(ReadEEPROM(EEAddrLCD_CardPolling_Line2) == 0xFF)
            {    
                // Default Text                
                printf_lcd(2,1,"  YR CARD PLZ   "); // Write to LCD Display 
            }
            else
            {
                // EEPROM Configuration String
                printf_lcd(2,1,"%s",abLCD_CardPolling_Line2); // Write to LCD Display 
            }
            
            
            return;
        }
        else
        {   
            //Verify CARD UID with LPC EEPROM UID LIST
            for(i=0; i < MAX_UID_LIST_ENTRIES;i++)
            {
                // Read actual UID List entry
                for(j=0; j < 8; j++)
                    baEE_UID_List_Entry[j] = ReadEEPROM(iEEAddr+j);

                // Compare actual UID with actual UID List entry 
                if(memcmp(abUID,baEE_UID_List_Entry,bUIDLen) == 0)
                {
                    // Card UID matches with current UID List entry
                    // ==> ACCESS
                    bAccessStatus = ALLOWED;
                    break; 
                }
                iEEAddr += UIDDataOffset;
            }   
            
            if(bAccessStatus == ALLOWED)
            {   
                // ACCESS ALLOWED
                OpenerOutput = 1;	// SET (Door) Opener Output pin  
                
                // Write to LCD Display Line 1 
                if(ReadEEPROM(EEAddrLCD_Auth_Card_Line1) == 0xFF)
                {   
                    // Default Display Content
                    printf_lcd(1,1,"     ACCESS     "); // Write to LCD Display 
                 }
                 else
                 {
                    // EEPROM Configuration String
                    printf_lcd(1,1,"%s",abLCD_Auth_Card_Line1); // Write to LCD Display 
                 }
                
                // Write to LCD Display Line 2
                if(ReadEEPROM(EEAddrLCD_Auth_Card_Line2) == 0xFF)
                {   
                    // Default Display Content
                    printf_lcd(2,1,"     ALLOWED    "); // Write to LCD Display 
                 }
                 else
                 {
                    // EEPROM Configuration String
                    printf_lcd(2,1,"%s",abLCD_Auth_Card_Line2); // Write to LCD Display 
                 }  
            }
            else
            {
                // ACCESS NOT ALLOWED

                OpenerOutput = 0;	// Clear (Door) Opener Output pin    

                // Write to LCD Display Line 1 
                if(ReadEEPROM(EEAddrLCD_Unauth_Card_Line1) == 0xFF)
                {  
                    printf_lcd(1,1,"  unauthorized  "); // Write to LCD Display 
                }
                else
                {
                    // EEPROM Configuration String
                    printf_lcd(1,1,"%s",abLCD_Unauth_Card_Line1); // Write to LCD Display 
                }

                // Write to LCD Display Line 2
                if(ReadEEPROM(EEAddrLCD_Unauth_Card_Line2) == 0xFF)
                {  
                    printf_lcd(2,1,"      CARD      "); // Write to LCD Display
                }
                else
                {
                    // EEPROM Configuration String
                    printf_lcd(2,1,"%s",abLCD_Unauth_Card_Line2); // Write to LCD Display 
                }
            }
        
        }
}

//------------------------------------------------------------------------
// Function: Read_EE_AccessControl_LCD_Content
//------------------------------------------------------------------------
// Description:
// This function read user defined LCD display strings form the EEPROM
// and stores the content in string variables.
//------------------------------------------------------------------------
void Read_EE_AccessControl_LCD_Content(void)
{
    unsigned char i;
    
    for(i=0; i < 16;i++)
    {
        abLCD_CardPolling_Line1[i] = ReadEEPROM(EEAddrLCD_CardPolling_Line1 + i);
        abLCD_CardPolling_Line2[i] = ReadEEPROM(EEAddrLCD_CardPolling_Line2 + i);

        abLCD_Auth_Card_Line1[i] = ReadEEPROM(EEAddrLCD_Auth_Card_Line1 + i);
        abLCD_Auth_Card_Line2[i] = ReadEEPROM(EEAddrLCD_Auth_Card_Line2 + i);

        abLCD_Unauth_Card_Line1[i] = ReadEEPROM(EEAddrLCD_Unauth_Card_Line1 + i);
        abLCD_Unauth_Card_Line2[i] = ReadEEPROM(EEAddrLCD_Unauth_Card_Line2 + i);
    }

    abLCD_CardPolling_Line1[16] = 0;
    abLCD_CardPolling_Line2[16] = 0;
    abLCD_Auth_Card_Line1[16] = 0;
    abLCD_Auth_Card_Line2[16] = 0;
    abLCD_Unauth_Card_Line1[16] = 0;
    abLCD_Unauth_Card_Line2[16] = 0;
}