//////////////////////////////////////////////////////////////////////////////
//
//                  ELEKTOR RFID READER for MIFARE and ISO14443-A
//
//                      Copyright (c) 2006 Gerhard H. Schalk
//         
//////////////////////////////////////////////////////////////////////////////
#include "stdio.h"
#include "string.h"
#include "RC522_Mifare.h"
#include "RC522_ErrCode.h"
#include "LCD.h"
#include "PC_Reader_COM.h"
#include "Mifare_Util.h"


//------------------------------------------------------------------------
// Function: MIFARE_Terminal
//------------------------------------------------------------------------
// Description:
// Mifare Reader Terminal Application Demo: 
// In this Mode the Reader polls for new Card and displays the Philips
// Mifare Type and the UID on the LCD Display. Basic card information is
// send via the RS232/USB serial Interface to the PC. 
// In case the Reader detects a Mifare Ultra Light card the whole 
// card memory content is read and send via RS232/USB to the PC.
//------------------------------------------------------------------------
void MIFARE_Terminal(void)
{
    unsigned char data abDataBuffer[16]; 
    unsigned char abUID[8], abOldUID[8],bSAK;
    unsigned char baATQ[2], bUIDLen, bFlagNewCard, bBlock,i;
    enum MifareICType eMifareType;
    short status;

    Rc522RFReset(5);  //RF - Reset 

    // Activate Card
    status = ActivateCard(ISO14443_3_REQA,baATQ,abUID,&bUIDLen,&bSAK);
    if(status !=  STATUS_SUCCESS)
    {
        // There is no card in the RF Field
        bFlagNewCard = 0;                   
        
        memset(abOldUID,0,sizeof(abOldUID)); // Clear abOldUID

        printf_lcd(1,1,"  RFID-Reader   "); // Write to LCD Display 
        printf_lcd(2,1,"  YR CARD PLZ   "); // Write to LCD Display 
        return;
    }
    else
    {   
        // At least one card is in the RF Field
        
        // Compare actual UID with previous one
        if(memcmp(abUID,abOldUID,sizeof(abUID)))
        {
            // New Card in the field 
            bFlagNewCard = 1;
            memcpy(abOldUID,abUID,sizeof(abOldUID));
        }
        else 
        {
            // We have activated a previous activated Card
             bFlagNewCard = 0;    
        }
    }

    // ----------------------------------------------------------------------------
    // Display the Card Information on the LCD
    // ----------------------------------------------------------------------------            

    // Check the Mifare IC Type with the Help of ATQA and SAK
    eMifareType = MIFARE_TypeIdentification(baATQ,bSAK);

    // Display Card Type Information on LCD
    switch(eMifareType)
    {
        case Unknown:
            printf_lcd(1,1,"  Unknown Card  "); 
            break;
        case Mifare_UL:
            printf_lcd(1,1,"  Mifare UL     "); 
            break;

        case Mifare_1K:
            printf_lcd(1,1,"  Mifare 1K     ");  
            break;

        case Mifare_4K:
            printf_lcd(1,1,"  Mifare 4K     ");   
            break;

        case Mifare_DesFire:
            printf_lcd(1,1,"  MF DesFire    ");
            break;
    }
    
    // Display Card UID on LCD
    if(bUIDLen == 4)    // Check for Single UID Card
    {
        printf_lcd(2,3,"%b02X%b02X%b02X%b02X    ",abUID[0],abUID[1],abUID[2],abUID[3]);
        printf_lcd(2,12,"         ");        
    }

    if(bUIDLen == 8)    // Check for Double UID Card
    {
        // abUID[0] is always 0x88 and will not be displayed
        printf_lcd(1,13,"%b02X",abUID[1]);        
        printf_lcd(2,3,"%b02X%b02X",abUID[2],abUID[3]);
        printf_lcd(2,7,"%b02X%b02X%b02X%b02X",abUID[4],abUID[5],abUID[6],abUID[7]);       
    }
   
    // ----------------------------------------------------------------------------
    // In case we have detected a new Card 
    //  ==> Send Card Information to Terminal via RS232/USB serial Interface
    // ----------------------------------------------------------------------------
    if(bFlagNewCard == 1)
    {
        switch(eMifareType)
        {
            case Unknown:
                printf("IC Type: Unknown IC Type\r\n"); 
                break;
            case Mifare_UL:
                printf("IC Type: Philips Mifare UltraLight\r\n");
                break;
            
            case Mifare_1K:
                printf("IC Type: Philips Mifare Standard 1K\r\n"); 
                break;
            
            case Mifare_4K:
                printf("IC Type: Philips Mifare Standard 4K\r\n"); 

                break;
            case Mifare_DesFire:
                printf("IC Type: Philips Mifare DesFire\r\n"); 
                break;
    }

        // Send Card UID 
        if(bUIDLen == 4)    // Check for Single UID Card
        {
            printf("Card UID: 0x%b02X %b02X %b02X %b02X\r\n",abUID[0],abUID[1],abUID[2],abUID[3]);
        }
        if(bUIDLen == 8)    // Check for Double UID Card
        {
            printf("Card UID: 0x%b02X %b02X %b02X %b02X %b02X %b02X %b02X %b02X\r\n",abUID[0],abUID[1],abUID[2],abUID[3],abUID[4],abUID[5],abUID[6],abUID[7]); 
        }
        printf("ATQA Byte[1]: 0x%b02X  ATQA Byte[0]: 0x%b02X  SAK: 0x%b02X\r\n\r\n",baATQ[1],baATQ[0],bSAK);


        // ----------------------------------------------------------------------------
        // In case we have detected a new MIFARE Ultra Light Card 
        //  ==> Send Card complete Data to Terminal via RS232/USB serial Interface
        // ----------------------------------------------------------------------------
        if(eMifareType == Mifare_UL)
        {
            // Read and send via RS232/USB UltraLight Block 0..3
            status = Read(0,abDataBuffer);
            if(status == STATUS_SUCCESS)
            {   
                printf("Serial Number [Page  0]: ");  PrintMifareULPage(abDataBuffer);
                printf("Serial Number [Page  1]: ");  PrintMifareULPage(&abDataBuffer[4]);
                printf("Internal/Lock [Page  2]: ");  PrintMifareULPage(&abDataBuffer[8]);
                printf("OTP           [Page  3]: ");  PrintMifareULPage(&abDataBuffer[12]);
            }
            
            // Read and send via RS232/USB UltraLight Data Block 4..15
            for(bBlock = 4; bBlock <13; bBlock+=4) 
            {    
                for(i=0; i<4; i++)
                {
                    status = Read(bBlock,abDataBuffer);
                    if(status == STATUS_SUCCESS)
                    {
                        printf("Data          [Page %2bu]: ", (bBlock+i));
                        PrintMifareULPage(&abDataBuffer[i*4]);
                    }
                }
            }
        }
    }
}