//////////////////////////////////////////////////////////////////////////////
//
//                  ELEKTOR RFID READER for MIFARE and ISO14443-A
//
//                      Copyright (c) 2006 Gerhard H. Schalk
//
//////////////////////////////////////////////////////////////////////////////
#include <REG936.H>
#include "Uart.h"   
#include "PC_Reader_COM.h"
#include "Mifare_PC_Reader.h"

//------------------------------------------------------------------------
// Function: Receive_PC_Command
//------------------------------------------------------------------------ 
// Description:
// This function is called from the UART Interrupt Service Routine 
// and receives a Command from the PC. After a complete command is received 
// the global Flag "gbNewCmdReceived" is set to '1'
//------------------------------------------------------------------------ 
// PC Command Fromat: (PC to Reader)
// gbaSerialRecBuffer: 0	 1	   2	  3     ..      n 
//                  --------------------------------------------
//                  | SOH | Cmd | Len | Data[0] .. Data[Len-1] |
//                  --------------------------------------------
//------------------------------------------------------------------------
void Receive_PC_Command(void)
{
    static unsigned char pRecBuffer;

    // Command State Machine
    switch(gbRecState)
    {
        case 0: //  Receive Start of Header
            gbaSerialRecBuffer[0] = SBUF;   

            // Check if we have received the Start of Header (SOH)
            if(gbaSerialRecBuffer[0] == COM_SOH)
            {   
            gbRecState = 1;
            }
            else
            {
                gbRecState = 0;
            }
            break;

        case 1: //  Receive Command Byte
            gbaSerialRecBuffer[1] = SBUF;  
            gbRecState = 2;
            break;

         case 2: //  Receive LEN Byte
            gbaSerialRecBuffer[2] = SBUF;
            
            if( gbaSerialRecBuffer[2] == 0)
            {
                gbRecState = 0;
                gbNewCmdReceived = SET;
            }
            else
            {   
                pRecBuffer = 0;
                gbRecState = 3;
             }
             break;
    
        case 3:  // Receive Data Bytes
              gbaSerialRecBuffer[(3 + pRecBuffer)] = SBUF;
              pRecBuffer++;

              // 
              if(pRecBuffer >= (gbaSerialRecBuffer[2])   )
              {
                gbRecState = 0;
                gbNewCmdReceived = SET;
              }
              break;
    }
}

//------------------------------------------------------------------------
// Function: Receive_PC_Command
//------------------------------------------------------------------------ 
// Description: This function sends a PC Command Response.
//------------------------------------------------------------------------ 
// Command Response Format: (Reader to PC)
//        SndBuffer:   0	   1	  2	      3     ..      n 
//                  -----------------------------------------------
//                  | SOH | Status | Len | Data[0] .. Data[Len-1] |
//                  -----------------------------------------------
//------------------------------------------------------------------------
void SendCmdResponce(unsigned char Status, unsigned char *SndBuffer, unsigned char SndLen)
{
    unsigned char i;
    unsigned char SndStatus;

    // Only the LOW Byte of the Mifare Cmd Status is returned
    // (The high Byte is fixed for each function there is not
    //  need to transmit this byte and simplifies the protocol)
    SndStatus = (unsigned char)Status&0x00FF;  

    putChar(COM_SOH);       // Send SOF .. Start of Frame
    putChar(SndStatus);     // Send STATUS
    
    if(Status == 0)
    {   
        putChar(SndLen);              //Send LEN
        for(i=0;i<SndLen;i++)
        {
            putChar(SndBuffer[i]);    //Send DATA
        }
    }
    else
    {
        putChar(0); // Send LEN in case STATUS != 0
    }
}
