//////////////////////////////////////////////////////////////////////////////
//
//                  ELEKTOR RFID READER for MIFARE and ISO14443-A
//
//                      Copyright (c) 2006 Gerhard H. Schalk
//
//////////////////////////////////////////////////////////////////////////////
#include <REG936.H>  

//-------------------------------------------------------------------------
//  void INI_Timer1_Delay_1ms (void)
//------------------------------------------------------------------------
// Description:
// Initializes the Timer 1 with following parameters
// Timer Mode 1: 16-Bit-Timer
//------------------------------------------------------------------------
void INI_Timer1(void)
{   
    //  Set Timer1 for 16-bit timer mode.  The
    TMOD = (TMOD & 0x0F) | 0x10;  // Timer 1 Mode 1 
    TF1 = 0;                      // Clear Timer 1 Flag
    TR1 = 0;                      // Stop Timer 1
}

//------------------------------------------------------------------------
// Function: Timer1_Delay_ms
//------------------------------------------------------------------------
// Description:
// Generates a delay of n x ms using the LPC Timer 1.
//------------------------------------------------------------------------
// Precondition: CPU must run with 16 MHz
//------------------------------------------------------------------------
// Parameters:
// IN  ms:  Delay Timer = multiple of ms
//------------------------------------------------------------------------
void Timer1_Delay_ms(unsigned int ms)
{   
    while(ms)
    {
        TF1 = 0;       // Clear Timer 1 Flag
        TH1 = 0xE0;    // 1 us
        TL1 = 0xBF;    
        TR1 = 1;       // Start Timer 1
        while(!TF1);
        TR1 = 0;       // Stop Timer 1
        ms--;
    }
}

//------------------------------------------------------------------------
// Function: Timer1_Delay_10us
//------------------------------------------------------------------------
// Description:
// Generates a delay of n x 10us using the LPC Timer 1.
//------------------------------------------------------------------------
// Precondition: CPU must run with 16 MHz
//------------------------------------------------------------------------
// Parameters:
// IN  ms:  Delay Timer = multiple of 10 us
//------------------------------------------------------------------------
#if 0 // This function is currently not used
void Timer1_Delay_10us(unsigned int n_10us)
{   
    while(n_10us)
    {
        TF1 = 0;	   // Clear Timer 1 Flag
        TH1 = 0xFF;    // 10 us
        TL1 = 0xC2;
        TR1 = 1;       // Start Timer 1
        while(!TF1);
        TR1 = 0;       // Stop Timer 1
        n_10us--;
    }
}
#endif