//////////////////////////////////////////////////////////////////////////////
//
//                  ELEKTOR RFID READER for MIFARE and ISO14443-A
//
//                      Copyright (c) 2006 Gerhard H. Schalk
//
//////////////////////////////////////////////////////////////////////////////
#include <REG936.H>   
#include "UART.h"
#include "PC_Reader_COM.h"
#include "Mifare_PC_Reader.h"

//------------------------------------------------------------------------
// Function: uart_init
//------------------------------------------------------------------------
// Description:
// Initializes the UART with following parameters:
// UART Mode1: 8-Bit Uart (8 bit, no parity, 1 stop bit)
//------------------------------------------------------------------------
// Parameters:
// IN  baudrate:  Baudarate (e.g. 115200)
// CCKL .. CPU Clock is defined in the Uart.h file
//------------------------------------------------------------------------
void uart_init(unsigned long  baudrate)
{
    unsigned int BRG;

    // Configure UART
    SCON   = 0x50;	// Mode1: 8-Bit Uart; enable serial reception

    // Configure Baud Rate Generator
    // For UART Baudrate generation the LPC independent 
    // Baud Rate Generator is used.
    // Baudrate Calculation:	 Baudrate = CCLK / ((BRG1,BRG0) + 16)

    BRGCON = 0x00;	// Stop Baudrate Generator

    BRG =   ((unsigned int)(CCKL / baudrate) - 16);
    BRGR0 = (unsigned char) (BRG & 0x00FF);
    BRGR1 = (unsigned char) (BRG >> 8);

    BRGCON = 0x03;

    // Configure UART Interrupt    
    RI = 0;         // Clear Uart Interrupt Flags
    TI = 0;

    EST = 0;       // Disable TX interrupt
    SSTAT |= 0x20; //Set CIDIS = 1 to separate Rx and Tx Interrupts
    ES = 1;        // Enable RX Interrupt
}

//------------------------------------------------------------------------
// Function: putChar
//------------------------------------------------------------------------
// Description:
// The putChar function transmits the character c using the serial port.
//------------------------------------------------------------------------
// Parameters:
// IN  c:       Character to output 
//------------------------------------------------------------------------
void putChar (unsigned char c)  
{
    SBUF = c;
    while(!TI);
    TI=0;
}

//------------------------------------------------------------------------
// Function: uart_isr
//------------------------------------------------------------------------
// Description:
// UART Interrupt Service Routine
//------------------------------------------------------------------------
void uart_isr(void ) interrupt 4 using 1
{
    if (RI)
    {
        RI = 0;
        gbMainState = MainState_PCReaderMode;
		Receive_PC_Command();
	}
}