//////////////////////////////////////////////////////////////////////////////
//
//                  ELEKTOR RFID READER for MIFARE and ISO14443-A
//
//                      Copyright (c) 2006 Gerhard H. Schalk
//
//                             Firmware Version V2.2
//
//////////////////////////////////////////////////////////////////////////////
#include <REG936.H>                 // LPC SFR definition
#include "stdio.h"                  // stdio Library
#include "LPC_Hardware.h"           // Output Pin definition
#include "UART.h"                   // Basic UART functions
#include "LCD.h"                    // Basic LCD functions
#include "Timer1.h"                 // Hardware delay Timer (LPC Timer1)
#include "EEPROM.h"
#include "RC522_Mifare.h"           // Basic ISO14443 & MIfARE functions
#include "PC_Reader_COM.h"          // Reader PC Communication
#include "Mifare_Terminal.h"        // Reader - Terminal Application Demo
#include "Mifare_PC_Reader.h"       // Reader - PC Reader Application Demo
#include "Mifare_Access_Control.h"  // Reader - Access Control Demo

void main(void)
{   
    OpenerOutput = 0;	// Clear (Door) Opener Output pin            
    
    //------------------------------------------------------------    
    //  Basic LPC936 Hardware Configuration
    //------------------------------------------------------------
    DIVM = 0x00;    // LPC clock divider
    TRIM &= ~0x40;  // no clock out on CLKOUT pin
    // select high power clock (CPU clock faster than 8MHz)
    AUXR1 &= ~0x80;
    
    //Set P0 Portpin; P0.0: Push-pull; P0.1-P0.7: Quasi-bidirectional
    P0M1 = 0x00;       
    P0M2 = 0x01; 
    
    // Portpin settings for I2C; Open-Drain
    P1M1 = 0x0C;       
    P1M2 = 0x8C;       
    
    // TxD = push-pull, RxD = input
    P1M1 &=  ~0x01;
    P1M2 |=   0x01;
    P1M1 |=   0x02;
    P1M2 &=  ~0x02;
   
    //------------------------------------------------------------------ 
    // Initialize Software States and Flags
    //------------------------------------------------------------------
    // Verify the LPC EEPROM "ACCESS CONTROL DEMO Enable" Flag
    if (ReadEEPROM(EEAddrAccessActFlag) ==  ACTIVE)
    {
        // Access Control Demo is active 
        // => Start Access Control Demo 
        gbMainState = MainState_ACCESS_CONTROL;  // Initialize  Main State
        Read_EE_AccessControl_LCD_Content(); 
    }
    else
    {
        // Access Control Demo is not active
        // => Start Terminal Mode Demo
        gbMainState = MainState_TerminalMode;  // Initialize  Main State 
    }
    gbRecState = 0; // Reset Serial (RS232/USB) Interrupt Receive State
    gbNewCmdReceived = CLEAR; // Clear Serial (RS232/USB) new Cmd Flag

    //------------------------------------------------------------------ 
    // Initialize Hardware
    //------------------------------------------------------------------
    INI_Timer1();       // Delay Timer (LPC Timer 1)
    uart_init(115200);  // Initialize Serial UART (RS232/USB)

    init_lcd();         // Initialize LCD Display 
    blank_lcd();        // Clear LCD Display
    
    printf_lcd(1,1,"  RFID-Reader   "); // Write to LCD Display 
    printf_lcd(2,1,"  Firmware 2.2  "); // Write to LCD Display
    Timer1_Delay_ms(1000);            // Display Version for 1 s

    Rc522Init();   // Initialize MFRC522 Reader IC
    EA = 1;        // Enable all Interrupts 

    while(1) // MAIN LOOP
    {   
        // Check Main State
        switch(gbMainState)     
        {
            // Reader Terminal Demo Mode
            case  MainState_TerminalMode:   
                uart_init(115200);
                MIFARE_Terminal();
                break;
            
            // PC Reader Demo Mode
            case MainState_PCReaderMode:    
                if(gbNewCmdReceived == SET)
                {
                    MIFARE_PC_Reader();

                    // Clear Serial (RS232/USB) new Cmd Flag
                    gbNewCmdReceived = CLEAR;
                }
                break;

            // Access Control Demo Mode           
            case MainState_ACCESS_CONTROL: 
                uart_init(115200);
                MIFARE_AccessControl();
                break;

            // Reader Terminal Demo Mode            
            default:   
                uart_init(115200);
                MIFARE_Terminal();
                break;
        }
    }
}   // End of Main
