Attribute VB_Name = "FTD2xxFunctions"
'//////////////////////////////////////////////////////////////////////////////
'//                                 MIFARE MAGIC
'//
'//                  ELEKTOR RFID READER for MIFARE and ISO14443-A
'//
'//                      Copyright (c) 2006 Gerhard H. Schalk
'//
'//////////////////////////////////////////////////////////////////////////////
'// Functions:   - Open_ReaderPort
'//              - Close_ReaderPort
'//              - Set_RS232_BaudRate
'//              - ReadUSBDataQueue
'//              - ReadUSBData
'//              - WriteUSBData
'//              - ExchangeUSBData
'//////////////////////////////////////////////////////////////////////////////
Option Explicit
Public glngHandle As Long  'Global FT Handle
Private Const mlngRecTimeOUT = 100 'Receive Timeout (100 Milliseconds)

'//------------------------------------------------------------------------
'// Function: Open_ReaderPort
'//------------------------------------------------------------------------
'// Description:
'//------------------------------------------------------------------------
'// Return:         Status
'// Open the USB Reader Port
'//------------------------------------------------------------------------
Public Function Open_ReaderPort() As Long
    Dim lngStatus As Long
    Dim LatencyTimer As Long
   
    lngStatus = FT_Open(0, glngHandle)
    If lngStatus <> FT_OK Then
        Open_ReaderPort = lngStatus
        Exit Function
    Else
        ' Set baud rate to 115200
        lngStatus = FT_SetBaudRate(glngHandle, 115200)
        
        If lngStatus <> FT_OK Then
            Open_ReaderPort = lngStatus
            Exit Function
        Else
            ' 8 data bits, 1 stop bit, no parity
            lngStatus = FT_SetDataCharacteristics(glngHandle, FT_BITS_8, FT_STOP_BITS_1, FT_PARITY_NONE)
            If lngStatus <> FT_OK Then
                Open_ReaderPort = lngStatus
                Exit Function
            End If
        End If
    End If
    
    'Set Transfer size to 64 byte for USB IN and OUT request
    lngStatus = FT_SetUSBParameters(glngHandle, 0, 0)
    If lngStatus <> FT_OK Then
        Open_ReaderPort = lngStatus
        Exit Function
    Else
        lngStatus = FT_OK
    End If
End Function

'//------------------------------------------------------------------------
'// Function: Close_ReaderPort
'//------------------------------------------------------------------------
'// Description:
'// Close an open USB Reader Port
'//------------------------------------------------------------------------
'// Return:         Status
'//------------------------------------------------------------------------
Public Function Close_ReaderPort() As Long
    Dim lngStatus As Long
    
    Close_ReaderPort = FT_Close(glngHandle)

End Function

'//------------------------------------------------------------------------
'// Function: Set_RS232_BaudRate
'//------------------------------------------------------------------------
'// Description:
'// This function sets the RS232 Reader Baud rate.
'// decimal numbers (0..9).
'//------------------------------------------------------------------------
'// Parameters:
'// IN Baudrate:    Baudrate (e.g. 115200)
'// Return:         Status
'//------------------------------------------------------------------------
Public Function Set_RS232_BaudRate(Baudrate As Long) As Long
    Set_RS232_BaudRate = FT_SetBaudRate(glngHandle, Baudrate)
End Function

'//------------------------------------------------------------------------
'// Function: ReadUSBDataQueue
'//------------------------------------------------------------------------
'// Description:
'// This function reads all characters from the USB data Queue
'//------------------------------------------------------------------------
'// Parameters:
'// IN  RecBuffer():   Receive Buffer
'// OUT RecLen:        Received Bytes
'//------------------------------------------------------------------------
Public Function ReadUSBDataQueue(ByRef RecBuffer() As Byte, ByRef RecLen As Long) As Long
    Dim lngStatus As Long
    Dim lngAmountInRxQueue As Long
    
    RecLen = 0
    lngAmountInRxQueue = 0
    
    'Get the number of characters in the receive queue
    lngStatus = FT_GetQueueStatus(glngHandle, lngAmountInRxQueue)
    If lngStatus <> FT_OK Then
        ReadUSBDataQueue = lngStatus
        Exit Function
    Else
        If lngAmountInRxQueue > 0 Then
            lngStatus = FT_Read(glngHandle, RecBuffer(0), lngAmountInRxQueue, RecLen)
            ReadUSBDataQueue = lngStatus
        End If
    End If
End Function

'//------------------------------------------------------------------------
'// Function: ReadUSBData
'//------------------------------------------------------------------------
'// Description:
'// This function reads characters from the USB data Queue
'//------------------------------------------------------------------------
'// Parameters:
'// IN:  TimeOut:           Read Timeout in milliseconds
'// IN: RecBuffer():        Receive Buffer
'// IN: BytesToRead:        Bytes expected to receive
'// OUT: RecLen:            Received Bytes
'//------------------------------------------------------------------------
Public Function ReadUSBData(TimeOut As Long, RecBuffer() As Byte, BytesToRead As Long, ByRef RecLen As Long) As Long
    Dim lngStatus As Long
    Dim lngAmountInRxQueue As Long
    
    RecLen = 0
    lngStatus = FT_SetTimeouts(glngHandle, TimeOut, 0)
    If lngStatus <> FT_OK Then
        ReadUSBData = lngStatus
        Exit Function
    Else
        lngStatus = FT_Read(glngHandle, RecBuffer(0), BytesToRead, RecLen)
        If lngStatus <> FT_OK Then
            ReadUSBData = lngStatus
        Else
            ReadUSBData = lngStatus
        End If
        
    End If
End Function
'//------------------------------------------------------------------------
'// Function: WriteUSBData
'//------------------------------------------------------------------------
'// Description:
'// This function transmit data via USB
'//------------------------------------------------------------------------
'// Parameters:
'// IN: SndBuffer():       Transmit Buffer
'// IN: SndLen:            Bytes to Transmit
'//------------------------------------------------------------------------
Public Function WriteUSBData(ByRef SndBuffer() As Byte, SndLen As Long) As Long
    Dim lngBytesWritten As Long
    WriteUSBData = FT_Write(glngHandle, SndBuffer(0), SndLen, lngBytesWritten)

End Function
'//------------------------------------------------------------------------
'// Function: ExchangeUSBData
'//------------------------------------------------------------------------
'// Description:
'// This function exchange data via USB
'//------------------------------------------------------------------------
'// Parameters:
'// IN: SndBuffer():       Transmit Buffer
'// IN: SndLen:            Bytes to Transmit
'// OUT: RecBuffer():      Receive Buffer
'// IN: BytesToRead:       Bytes Expected to Receive
'// OUT: RecLen:           Received Bytes
'// Return:                Status
'//------------------------------------------------------------------------
Public Function ExchangeUSBData(ByRef SndBuffer() As Byte, SndLen As Long, ByRef RecBuffer() As Byte, BytesToRead As Long, ByRef RecLen As Long) As Long
    Dim lngStatus As Long
    Dim strDebug As String
        If MDIForm_main.mnu_LogSerialDebug.Checked = True Then
            
            'Write Serial Debug Info to Form_MifareUltraLight.Text_Log
            strDebug = BytArray2HEXString(SndBuffer, SndLen, True)
            Form_MifareUltraLight.Text_Log.SelText = " (Serial Debug Snd[" + Str$(SndLen) + "]:" + strDebug
            'Debug.Print "Snd[" + Str$(SndLen) + "]:" + strDebug
        End If
    If SndLen > 0 Then
        lngStatus = WriteUSBData(SndBuffer, SndLen)
        If lngStatus <> FT_OK Then
            ExchangeUSBData = lngStatus
            Exit Function
        End If
    End If
        
        lngStatus = ReadUSBData(mlngRecTimeOUT, RecBuffer, BytesToRead, RecLen)
        If lngStatus <> FT_OK Then
            ExchangeUSBData = lngStatus          'Read failed
            Exit Function
        Else
            If RecLen <> BytesToRead Then
                ExchangeUSBData = FT_RecTimeOut ' Timeout occured
            Else
                ExchangeUSBData = lngStatus     ' OK
            End If
        End If
        
        If MDIForm_main.mnu_LogSerialDebug.Checked = True Then
            'Write Serial Debug Info to Form_MifareUltraLight.Text_Log
            strDebug = BytArray2HEXString(RecBuffer, RecLen, True)
            Form_MifareUltraLight.Text_Log.SelText = " Rcv[" + Str$(RecLen) + "]: " + strDebug + ") "
            'Debug.Print "  Rcv[" + Str$(RecLen) + "]: " + strDebug + ") "
        End If
End Function
