Attribute VB_Name = "PC_Reader_MIFARE_Functions"
'//////////////////////////////////////////////////////////////////////////////
'//                                 MIFARE MAGIC
'//
'//                  ELEKTOR RFID READER for MIFARE and ISO14443-A
'//
'//                      Copyright (c) 2006 Gerhard H. Schalk
'//
'//////////////////////////////////////////////////////////////////////////////
'// Functions:   - ChangeReaderMode
'//              - Change_RS232_Baudrate
'//              - ReaderRF_Reset
'//              - ISO14443_ActivateCard
'//              - ISO14443_Request
'//              - ISO14443_Anticollision
'//              - ISO14443_Select
'//              - ISO14443_Halt
'//              - ISO14443_MifareRead
'//              - ISO14443_UL_MifareWrite
'//////////////////////////////////////////////////////////////////////////////
Option Explicit
'//------------------------------------------------------------------------
'//  PC Reader Command definitions
'//------------------------------------------------------------------------
'PC Reader - Reader Commands
Public Const READER_TERMINAL_MODE = &HF0
Public Const READER_PC_READER_MODE = &HF1
Public Const READER_CHANGE_BAUDRATE = &HF2
Public Const READER_ACCESS_CONTROL_MODE = &HF3
Public Const READER_LPC_EEPROM_WRITE = &HF5
Public Const READER_LPC_EEPROM_READ = &HF6
Public Const READER_LPC_SOFT_RESET = &HFF
Public Const READER_RF_RESET = &H20

'PC Reader - Mifare Commands
Public Const MIFARE_ATICATECARD_REQA = &H40
Public Const MIFARE_ATICATECARD_WUPA = &H41
Public Const MIFARE_ATICATECARD_RF_RST = &H42

Public Const ISO14443_REQA = &H26
Public Const ISO14443_WUPA = &H52

Public Const ISO14443_Anticol_1 = &H93
Public Const ISO14443_Select_1 = &H94
Public Const ISO14443_Anticol_2 = &H95
Public Const ISO14443_Select_2 = &H96

Public Const ISO14443_HALTA = &H50

Public Const MIFARE_READ = &H30
Public Const MIFARE_WRITE = &HA0

'//------------------------------------------------------------------------
'PC Reader - Mifare Error Codes
'//------------------------------------------------------------------------
Public Const MIFARE_OK = &H0
Public Const MIFARE_Err_FT_USB = &H1F00
Public Const MIFARE_Err_SerResFrame = &HF00
Public Const MIFARE_Err_TimeOut = &H1
Public Const MIFARE_Err_Protocol = &HB
'//------------------------------------------------------------------------
'// Function: ChangeReaderMode
'//------------------------------------------------------------------------
'// Description: This function changes the Reader Mode between
'//              Terminal and PC_Reader-Mode
'//------------------------------------------------------------------------
'// Parameters:
'// IN: CmdCode: READER_TERMINAL_MODE or READER_PC_READER_MODE
'// Return: Status
'//------------------------------------------------------------------------
Public Function ChangeReaderMode(CmdCode As Byte) As Long
    Dim bytRcvBuffer(16) As Byte
    Dim bytSndBuffer(16) As Byte
    Dim bytRecStatus As Byte
    Dim lngStatus As Long
    Dim lngRecLen As Long
     
    lngRecLen = 0
    lngStatus = PCReaderCmdExchange(CmdCode, bytSndBuffer, 0, bytRcvBuffer(), 0, lngRecLen)
    ChangeReaderMode = lngStatus 'Return Error Code
End Function
'//------------------------------------------------------------------------
'// Function: Change_RS232_Baudrate
'//------------------------------------------------------------------------
'// Description:
'// This function changes the PC Reader RS232/USB Baudrate
'//------------------------------------------------------------------------
'// Precondition: Reader in PC_Reader Mode
'//------------------------------------------------------------------------
'// Parameters:
'// IN: Baudrate: Baudrate (e.g. 9600, 115200,..)
'// Return: Status
'//------------------------------------------------------------------------
Public Function Change_RS232_Baudrate(Baudrate As Long) As Long
    Dim bytRcvBuffer(16) As Byte
    Dim bytSndBuffer(16) As Byte
    Dim bytRecStatus As Byte
    Dim lngStatus As Long
    Dim lngRecLen As Long
     
    Select Case Baudrate
        Case 9600:      bytSndBuffer(0) = 0
        Case 19200:     bytSndBuffer(0) = 1
        Case 38400:     bytSndBuffer(0) = 2
        Case 57600:     bytSndBuffer(0) = 3
        Case 115200:    bytSndBuffer(0) = 4
        Case 230400:    bytSndBuffer(0) = 5
        Case 460800:    bytSndBuffer(0) = 6
        Case Else:      bytSndBuffer(0) = 4
    End Select
     
    lngRecLen = 0
    lngStatus = PCReaderCmdExchange(READER_CHANGE_BAUDRATE, bytSndBuffer, 1, bytRcvBuffer(), 0, lngRecLen)
    
        Select Case Baudrate
        Case 9600:      Set_RS232_BaudRate (9600)
        Case 19200:     Set_RS232_BaudRate (19200)
        Case 38400:     Set_RS232_BaudRate (38400)
        Case 57600:     Set_RS232_BaudRate (57600)
        Case 115200:    Set_RS232_BaudRate (115200)
        Case 230400:    Set_RS232_BaudRate (230400)
        Case 460800:    Set_RS232_BaudRate (460800)
        Case Else:      Set_RS232_BaudRate (115200)
    End Select
    
    Change_RS232_Baudrate = lngStatus 'Return Error Code
End Function
'//------------------------------------------------------------------------
'// Function: Reader_LPC_EE_Read
'//------------------------------------------------------------------------
'// Description:
'// This function reads 1 byte of the LPC936 EEPROM (Configuration) Data.
'// EEPROM Addr Range is 0 .. 0x01FF
'//------------------------------------------------------------------------
'// Precondition: Reader in PC_Reader Mode
'//------------------------------------------------------------------------
'// Return: Status
'//------------------------------------------------------------------------
Public Function Reader_LPC_EE_Read(EEAddr As Integer, ByRef Data As Byte) As Long
    Dim bytRcvBuffer(16) As Byte
    Dim bytSndBuffer(16) As Byte
    Dim bytRecStatus As Byte
    Dim lngStatus As Long
    Dim lngRecLen As Long
     
    lngRecLen = 0
    bytSndBuffer(0) = (EEAddr And &H100) / 256
    bytSndBuffer(1) = (EEAddr And &HFF)
    lngStatus = PCReaderCmdExchange(READER_LPC_EEPROM_READ, bytSndBuffer, 2, bytRcvBuffer(), 1, lngRecLen)
    If lngStatus = MIFARE_OK Then
        Data = bytRcvBuffer(0)
    End If
    Reader_LPC_EE_Read = lngStatus 'Return Error Code
End Function
'//------------------------------------------------------------------------
'// Function: Reader_LPC_EE_Write
'//------------------------------------------------------------------------
'// Description:
'// This function reads 1 byte of the LPC936 EEPROM (Configuration) Data.
'// EEPROM Addr Range is 0 .. 0x01FF
'//------------------------------------------------------------------------
'// Precondition: Reader in PC_Reader Mode
'//------------------------------------------------------------------------
'// Return: Status
'//------------------------------------------------------------------------
Public Function Reader_LPC_EE_Write(EEAddr As Integer, Data As Byte) As Long
    Dim bytRcvBuffer(16) As Byte
    Dim bytSndBuffer(16) As Byte
    Dim bytRecStatus As Byte
    Dim lngStatus As Long
    Dim lngRecLen As Long
     
    lngRecLen = 0
    bytSndBuffer(0) = (EEAddr And &H100) / 256
    bytSndBuffer(1) = (EEAddr And &HFF)
    bytSndBuffer(2) = Data
    lngStatus = PCReaderCmdExchange(READER_LPC_EEPROM_WRITE, bytSndBuffer, 3, bytRcvBuffer(), 0, lngRecLen)
    Reader_LPC_EE_Write = lngStatus 'Return Error Code
End Function

'//------------------------------------------------------------------------
'// Function: ReaderRF_Reset
'//------------------------------------------------------------------------
'// Description:
'// This function turns off the RF-Field for 5 ms. Elapsing this time
'// the RF-Field is turned on.
'//------------------------------------------------------------------------
'// Precondition: Reader in PC_Reader Mode
'//------------------------------------------------------------------------
'// Return: Status
'//------------------------------------------------------------------------
Public Function ReaderRF_Reset() As Long
    Dim bytRcvBuffer(16) As Byte
    Dim bytSndBuffer(16) As Byte
    Dim bytRecStatus As Byte
    Dim lngStatus As Long
    Dim lngRecLen As Long
     
    lngRecLen = 0
    lngStatus = PCReaderCmdExchange(READER_RF_RESET, bytSndBuffer, 0, bytRcvBuffer(), 0, lngRecLen)
    ReaderRF_Reset = lngStatus 'Return Error Code
End Function
'//------------------------------------------------------------------------
'// Function: ISO14443_ActivateCard
'//------------------------------------------------------------------------
'// Description:
'// This Function performs a REQA or WUPA, Anticollision, Select sequence
'// to activate a single or double UID card.
'//------------------------------------------------------------------------
'// Precondition: Reader in PC_Reader Mode
'//------------------------------------------------------------------------
'// Parameters:
'// IN:  CmdCode: command code(ISO14443_REQA or ISO14443_WUPA)
'// OUT: ATQA():  2 byte buffer to save the card Answer to Request (ATQA)
'// OUT: UID():   8 byte buffer to save the card UID
'// OUT: UIDLen:  1 byte to save Card UID Length; (uid_len = 4 for
'//                 Single UID card  or uid_len = 8 for Double UID card)
'// OUT: SAK:     1 byte to save the card Select Acknowledge (SAK) info
'// Return: Status
'//------------------------------------------------------------------------
Public Function ISO14443_ActivateCard(CmdCode As Byte, ByRef ATQA() As Byte, _
                                      ByRef UID() As Byte, ByRef UIDLen As Byte, _
                                      ByRef SAK As Byte) As Long
    Dim bytRcvBuffer(16) As Byte
    Dim bytSndBuffer(16) As Byte
    Dim bytRecStatus As Byte
    Dim lngStatus As Long
    Dim lngRecLen As Long
     
    lngRecLen = 0
    lngStatus = PCReaderCmdExchange(CmdCode, bytSndBuffer, 0, bytRcvBuffer(), 12, lngRecLen)
    If lngStatus = MIFARE_OK Then
        ATQA(0) = bytRcvBuffer(0)
        ATQA(1) = bytRcvBuffer(1)
               
        UID(0) = bytRcvBuffer(2)
        UID(1) = bytRcvBuffer(3)
        UID(2) = bytRcvBuffer(4)
        UID(3) = bytRcvBuffer(5)
        UID(4) = bytRcvBuffer(6)
        UID(5) = bytRcvBuffer(7)
        UID(6) = bytRcvBuffer(8)
        UID(7) = bytRcvBuffer(9)
        UIDLen = bytRcvBuffer(10)
        SAK = bytRcvBuffer(11)
    End If
    ISO14443_ActivateCard = lngStatus 'Return Error Code
End Function
'//------------------------------------------------------------------------
'// Function: ISO14443_Request
'//------------------------------------------------------------------------
'// Description:
'// This function performs a ISO14443 REQA or WUPA command.
'//------------------------------------------------------------------------
'// Precondition: Reader in PC_Reader Mode
'//------------------------------------------------------------------------
'// Parameters:
'// IN CmdCode:   ISO14443_REQA or ISO14443_WUPA
'// OUT: ATQA():  2 byte buffer to save the card Answer to Request (ATQA)
'// Return: Status
'//------------------------------------------------------------------------
Public Function ISO14443_Request(CmdCode As Byte, ByRef ATQA() As Byte) As Long
    Dim bytRcvBuffer(16) As Byte
    Dim bytSndBuffer(16) As Byte
    Dim bytRecStatus As Byte
    Dim lngStatus As Long
    Dim lngRecLen As Long
     
    lngRecLen = 0
    lngStatus = PCReaderCmdExchange(CmdCode, bytSndBuffer, 0, bytRcvBuffer(), 2, lngRecLen)
    If lngStatus = MIFARE_OK Then
        ATQA(0) = bytRcvBuffer(0)
        ATQA(1) = bytRcvBuffer(1)
    End If
    ISO14443_Request = lngStatus 'Return Error Code
End Function
'//------------------------------------------------------------------------
'// Function: ISO14443_Anticollision
'//------------------------------------------------------------------------
'// Description:
'// This function performs a ISO14443 Anticollion Cascade Level 1 or
'// Anticollion Cascade Level 2 command
'//------------------------------------------------------------------------
'// Precondition: Reader in PC_Reader Mode
'//------------------------------------------------------------------------
'// Parameters:
'// IN CmdCode: ISO14443_Anticol_1 or ISO14443_Anticol_2
'// OUT: UID():   8 byte buffer to save the card UID
'// Return: Status
'//------------------------------------------------------------------------
Public Function ISO14443_Anticollision(CmdCode As Byte, ByRef UID() As Byte) As Long
    Dim bytRcvBuffer(16) As Byte
    Dim bytSndBuffer(16) As Byte
    Dim bytRecStatus As Byte
    Dim lngStatus As Long
    Dim lngRecLen As Long
     
    lngRecLen = 0
    lngStatus = PCReaderCmdExchange(CmdCode, bytSndBuffer, 0, bytRcvBuffer(), 4, lngRecLen)
    If lngStatus = MIFARE_OK Then
        UID(0) = bytRcvBuffer(0)
        UID(1) = bytRcvBuffer(1)
        UID(2) = bytRcvBuffer(2)
        UID(3) = bytRcvBuffer(3)
    End If
    ISO14443_Anticollision = lngStatus 'Return Error Code
End Function
'//------------------------------------------------------------------------
'// Function: ISO14443_Select
'//------------------------------------------------------------------------
'// Description:
'// This function performs a ISO14443 Select Cascade Level 1 or
'// Select Cascade Level 2 command and selects the card by the
'// specified card UID number.
'//------------------------------------------------------------------------
'// Precondition: Reader in PC_Reader Mode
'//------------------------------------------------------------------------
'// Parameters:
'// IN CmdCode: ISO14443_Anticol_1 or ISO14443_Anticol_2
'// IN: UID():   8 byte buffer Card UID
'// OUT: SAK:     1 byte to save the card Select Acknowledge (SAK) info
'// Return: Status
'//------------------------------------------------------------------------
Public Function ISO14443_Select(CmdCode As Byte, UID() As Byte, ByRef SAK() As Byte) As Long
    Dim bytRcvBuffer(16) As Byte
    Dim bytRecStatus As Byte
    Dim lngStatus As Long
    Dim lngRecLen As Long
     
    lngRecLen = 0
    lngStatus = PCReaderCmdExchange(CmdCode, UID(), 4, bytRcvBuffer(), 1, lngRecLen)
    If lngStatus = MIFARE_OK Then
        SAK(0) = bytRcvBuffer(0)
    End If
    ISO14443_Select = lngStatus 'Return Error Code
End Function
'//------------------------------------------------------------------------
'// Function: ISO14443_Halt
'//------------------------------------------------------------------------
'// Description:
'// This function performs a ISO14443 HALTA command.
'//------------------------------------------------------------------------
'// Precondition: Reader in PC_Reader Mode
'//------------------------------------------------------------------------
'// Return: Status
'//------------------------------------------------------------------------
Public Function ISO14443_Halt() As Long
    Dim bytRcvBuffer(16) As Byte
    Dim bytSndBuffer(16) As Byte
    Dim bytRecStatus As Byte
    Dim lngStatus As Long
    Dim lngRecLen As Long
     
    lngRecLen = 0
    lngStatus = PCReaderCmdExchange(ISO14443_HALTA, bytSndBuffer, 0, bytRcvBuffer(), 0, lngRecLen)
    ISO14443_Halt = lngStatus 'Return Error Code
End Function
'//------------------------------------------------------------------------
'// Function: ISO14443_MifareRead
'//------------------------------------------------------------------------
'// Description:
'// This function peroforms a MIFARE UL or MIFARE 1/4K Read command
'//------------------------------------------------------------------------
'// Precondition: Reader in PC_Reader Mode
'//------------------------------------------------------------------------
'// Parameters:
'// IN Block: MIFARE UL Page Address / MIFARE 1/4K Block Number
'// OUT ReadData(): 16 byte data buffer
'// Return: Status
'//------------------------------------------------------------------------
Public Function ISO14443_MifareRead(Block As Byte, ByRef ReadData() As Byte) As Long
    Dim bytRcvBuffer(20) As Byte
    Dim bytSndBuffer(16) As Byte
    Dim bytRecStatus As Byte
    Dim lngStatus As Long
    Dim lngRecLen As Long
    
    bytSndBuffer(0) = Block
    lngRecLen = 0
    lngStatus = PCReaderCmdExchange(MIFARE_READ, bytSndBuffer, 1, ReadData(), 16, lngRecLen)
    ISO14443_MifareRead = lngStatus 'Return Error Code
End Function
'//------------------------------------------------------------------------
'// Function: ISO14443_UL_MifareWrite
'//------------------------------------------------------------------------
'// Description:
'// This function peroforms a MIFARE UL COMPATIBILTY WRITE command
'//------------------------------------------------------------------------
'// Precondition: Reader in PC_Reader Mode
'//------------------------------------------------------------------------
'// Parameters:
'// IN Block: MIFARE UL Page Address / MIFARE 1/4K Block Number
'// OUT ReadData(): 4 byte data buffer
'// Return: Status
'//------------------------------------------------------------------------
Public Function ISO14443_UL_MifareWrite(Page As Byte, ByRef WriteData() As Byte) As Long
    Dim bytSndBuffer(20) As Byte
    Dim bytRcvBuffer(16) As Byte
    Dim bytRecStatus As Byte
    Dim lngStatus As Long
    Dim lngRecLen As Long
    
    bytSndBuffer(0) = Page
    bytSndBuffer(1) = WriteData(0)
    bytSndBuffer(2) = WriteData(1)
    bytSndBuffer(3) = WriteData(2)
    bytSndBuffer(4) = WriteData(3)
    'When using Mifare compatibility WRITE it is recommended to set the
    'remaining bytes 4 to 15 to all '0'
    bytSndBuffer(5) = 0
    bytSndBuffer(6) = 0
    bytSndBuffer(7) = 0
    bytSndBuffer(8) = 0
    
    bytSndBuffer(9) = 0
    bytSndBuffer(10) = 0
    bytSndBuffer(11) = 0
    bytSndBuffer(12) = 0
    
    bytSndBuffer(13) = 0
    bytSndBuffer(14) = 0
    bytSndBuffer(15) = 0
    bytSndBuffer(16) = 0
    
    lngRecLen = 0
    lngStatus = PCReaderCmdExchange(MIFARE_WRITE, bytSndBuffer, 17, bytRcvBuffer(), 0, lngRecLen)
    ISO14443_UL_MifareWrite = lngStatus 'Return Error Code
End Function



























