Attribute VB_Name = "MifareUTILFunctions"
'//////////////////////////////////////////////////////////////////////////////
'//                                 MIFARE MAGIC
'//
'//                  ELEKTOR RFID READER for MIFARE and ISO14443-A
'//
'//                      Copyright (c) 2006 Gerhard H. Schalk
'//
'//////////////////////////////////////////////////////////////////////////////
'// Functions:   - Get_MIFARE_TypeIdentification_String
'//              - MIFARE_TypeIdentification
'//////////////////////////////////////////////////////////////////////////////
Option Explicit

Public Const ATQA_Philips_Mifare_UL = &H4400  ' ATQA Byte 0 | ATQA Byte 1
Public Const ATQA_Philips_Mifare_1K = &H400   ' ATQA Byte 0 | ATQA Byte 1
Public Const ATQA_Philips_Mifare_4K = &H200   ' ATQA Byte 0 | ATQA Byte 1
Public Const ATQA_Philips_DesFire = &H4403    ' ATQA Byte 0 | ATQA Byte 1
    
Public Const SAK_Philips_Mifare_UL = &H0
Public Const SAK_Philips_Mifare_1K = &H8
Public Const SAK_Philips_Mifare_4K = &H18
Public Const SAK_Philips_DesFire = &H20

Public Enum MifareICType
    Unknown
    Mifare_UL
    Mifare_1K
    Mifare_4K
    Mifare_DesFire
End Enum
'//------------------------------------------------------------------------
'// Function: Get_MIFARE_TypeIdentification_String
'//------------------------------------------------------------------------
'// Description:
'// This function delivers the chip type information for all
'// Philips MIFARE ICs
'//------------------------------------------------------------------------
'// Parameters:
'// IN: *baATQ: 2 byte buffer; Card Answer to Request (ATQA)
'// IN: *sak:   1 byte; Card Select Acknowledge (SAK)
'// Return:     String of the Mifare IC Type
'//------------------------------------------------------------------------
Public Function Get_MIFARE_TypeIdentification_String(ATQA() As Byte, SAK As Byte) As String
    Dim MifareType As MifareICType
    
    MifareType = MIFARE_TypeIdentification(ATQA(), SAK)
    
    Select Case MifareType
        Case Unknown:
            Get_MIFARE_TypeIdentification_String = "Unknown IC Type"
        Case Mifare_UL:
            Get_MIFARE_TypeIdentification_String = "Philips Mifare Ultra Light"
        Case Mifare_1K:
            Get_MIFARE_TypeIdentification_String = "Philips Mifare 1K"
        Case Mifare_4K:
            Get_MIFARE_TypeIdentification_String = "Philips Mifare 4K"
        Case Mifare_DesFire:
            Get_MIFARE_TypeIdentification_String = "Philips Mifare DesFire"
    End Select
End Function
'//------------------------------------------------------------------------
'// Function: MIFARE_TypeIdentification
'//------------------------------------------------------------------------
'// Description:
'// This function delivers the chip type information for all
'// Philips MIFARE ICs
'//------------------------------------------------------------------------
'// Parameters:
'// IN: *baATQ: 2 byte buffer; Card Answer to Request (ATQA)
'// IN: *sak:   1 byte; Card Select Acknowledge (SAK)
'// Return: MifareICType (Unknown, Mifare_UL, Mifare_1K, Mifare_4K or
'//                       Mifare_DesFire)
'//------------------------------------------------------------------------
Public Function MIFARE_TypeIdentification(ATQA() As Byte, SAK As Byte) As MifareICType
    
    Dim iATQA As Integer
    
    iATQA = ATQA(0) * 256 + ATQA(1)
        
    If ((iATQA = ATQA_Philips_Mifare_UL) And (SAK = SAK_Philips_Mifare_UL)) Then
        MIFARE_TypeIdentification = Mifare_UL
    ElseIf ((iATQA = ATQA_Philips_Mifare_1K) And (SAK = SAK_Philips_Mifare_1K)) Then
        MIFARE_TypeIdentification = Mifare_1K
    ElseIf ((iATQA = ATQA_Philips_Mifare_4K) And (SAK = SAK_Philips_Mifare_4K)) Then
        MIFARE_TypeIdentification = Mifare_4K
    ElseIf ((iATQA = ATQA_Philips_DesFire) And (SAK = SAK_Philips_DesFire)) Then
        MIFARE_TypeIdentification = Mifare_DesFire
    Else
        MIFARE_TypeIdentification = Unknown
    End If

End Function















