Attribute VB_Name = "PC_Reader_Cmd_Exchange"
'//////////////////////////////////////////////////////////////////////////////
'//                                 MIFARE MAGIC
'//
'//                  ELEKTOR RFID READER for MIFARE and ISO14443-A
'//
'//                      Copyright (c) 2006 Gerhard H. Schalk
'//
'//////////////////////////////////////////////////////////////////////////////
'// Functions:   - PCReaderCmdExchange
'//////////////////////////////////////////////////////////////////////////////
Option Explicit
'//------------------------------------------------------------------------
'// Function: PCReaderCmdExchange
'//------------------------------------------------------------------------
'// Description: This function exchange a PC Reader Command
'//------------------------------------------------------------------------
'// Parameters:
'// IN: CmdCode: PC Reader Cmd Code (e.g.READER_TERMINAL_MODE,ISO14443_REQA)
'// IN: SndCmdData():       Transmit Buffer
'// IN: SndCmdDataLen:      Bytes to Transmit
'// OUT: RecData():         Receive Buffer
'// IN: ExpectedRcvDataLen: Bytes Expected to Receive
'// OUT: RecDataLen:        Received Bytes
'// Return:                 Status
'//------------------------------------------------------------------------
'// Function Internal Cmd Format:
'// PC Command: (PC to Reader)
'// bytSndBuffer         0     1     2      3     ..      n
'//                  --------------------------------------------
'//                  | SOH | Cmd | Len | Data[0] .. Data[Len-1] |
'//                  --------------------------------------------
'//------------------------------------------------------------------------
'// Command Response:  (Reader to PC)
'//
'//  bytRcvBuffer        0       1      2       3     ..      n
'//                  -----------------------------------------------
'//                  | SOH | Status | Len | Data[0] .. Data[Len-1] |
'//                  -----------------------------------------------
'//------------------------------------------------------------------------
Public Function PCReaderCmdExchange(CmdCode As Byte, _
                                    SndCmdData() As Byte, SndCmdDataLen As Long, _
                                    ByRef RecData() As Byte, ExpectedRcvDataLen As Long, _
                                    ByRef RecDataLen As Long) As Long
    
    Const SNDREC_BUFFER_SIZE = 300
    Const SOF = &HA5
    Const SERIAL_CMD_HEADER_LEN = 3
        
    Dim bytSndBuffer(SNDREC_BUFFER_SIZE) As Byte
    Dim bytRcvBuffer(SNDREC_BUFFER_SIZE) As Byte
    Dim bytRecStatus As Byte
    Dim lngStatus As Long
    Dim lngRecLen As Long
    Dim lngLoopCounter As Long
             
    'PC => Reader Serial Command Frame
    bytSndBuffer(0) = SOF           'Start of Frame
    bytSndBuffer(1) = CmdCode     'Mifare Command Byte
    bytSndBuffer(2) = SndCmdDataLen 'Data LEN Byte
    
    'Copy Mifare Command Data into bytSndBuffer
    If SndCmdDataLen > SNDREC_BUFFER_SIZE - 3 Then
        SndCmdDataLen = SNDREC_BUFFER_SIZE - 3
    End If
    
    For lngLoopCounter = 0 To SndCmdDataLen
        bytSndBuffer(lngLoopCounter + SERIAL_CMD_HEADER_LEN) = SndCmdData(lngLoopCounter)
    Next lngLoopCounter
    
    RecDataLen = 0
    lngRecLen = 0
    lngStatus = ExchangeUSBData(bytSndBuffer, SERIAL_CMD_HEADER_LEN + SndCmdDataLen, bytRcvBuffer(), _
                                   SERIAL_CMD_HEADER_LEN + ExpectedRcvDataLen, lngRecLen)
                                   
    'Check for FTDI Error (USB Error)
    If (lngStatus <> FT_OK) And (lngStatus <> FT_RecTimeOut) Then
        PCReaderCmdExchange = MIFARE_Err_FT_USB 'Return FTDI Error
        Exit Function
    Else
        'Check if we have received the correct number of bytes
        If (ExpectedRcvDataLen + SERIAL_CMD_HEADER_LEN) = lngRecLen Then
            'Check for correct Reader => PC  Serial Responce Frame
            If (bytRcvBuffer(0) = SOF) And (bytRcvBuffer(2) = (lngRecLen - SERIAL_CMD_HEADER_LEN)) Then
                RecDataLen = lngRecLen - SERIAL_CMD_HEADER_LEN
                
                For lngLoopCounter = 0 To (lngRecLen - SERIAL_CMD_HEADER_LEN)
                    RecData(lngLoopCounter) = bytRcvBuffer(lngLoopCounter + SERIAL_CMD_HEADER_LEN)
                Next lngLoopCounter
                
                PCReaderCmdExchange = bytRcvBuffer(1) 'Correct Frame Received / Return Mifare Error Code
                Exit Function
            End If
            
        Else
            'Check if we have received an corrrect Serial Responce Error Frame
            If (bytRcvBuffer(0) = SOF) And (bytRcvBuffer(2) = 0) Then
                PCReaderCmdExchange = bytRcvBuffer(1) 'Return Mifare Error Code
                Exit Function
            Else
                PCReaderCmdExchange = MIFARE_Err_SerResFrame 'Serial Responce Frame Error
                Exit Function
            End If
        End If
    End If
End Function

