Attribute VB_Name = "UTIL_Functions"
'//////////////////////////////////////////////////////////////////////////////
'//                                 MIFARE MAGIC
'//
'//                  ELEKTOR RFID READER for MIFARE and ISO14443-A
'//
'//                      Copyright (c) 2006 Gerhard H. Schalk
'//
'//////////////////////////////////////////////////////////////////////////////
'// Functions:   - EnterOnlyNumberKeys
'//              - EnterOnlyHexKeys
'//              - Hex2Dec
'//              - BytArray2HEXString
'//              - BytArray2ASCIIString
'//////////////////////////////////////////////////////////////////////////////
Option Explicit
'//------------------------------------------------------------------------
'// Function: RemoveAllSpaces
'//------------------------------------------------------------------------
'// Description:
'// This function removes all spaces within a string
'//------------------------------------------------------------------------
'// Parameters:
'// IN str: Input String
'// Return: Formated String
'//------------------------------------------------------------------------
Function RemoveAllSpaces(str As String) As String
    Dim StrLen, i As Integer
    Dim strTemp As String
    
    'Leerzeichen entfernen
    StrLen = Len(str)
    For i = 1 To StrLen
         strTemp = Mid$(str, i, 1)
        If (strTemp <> " ") Then
            RemoveAllSpaces = RemoveAllSpaces & strTemp
        End If
    Next i

End Function

'//------------------------------------------------------------------------
'// Function: EnterOnlyNumberKeys
'//------------------------------------------------------------------------
'// Description:
'// This function used for text boxes to allow only to enter
'// decimal numbers (0..9).
'//------------------------------------------------------------------------
'// Parameters:
'// IN KeyAscii:
'//------------------------------------------------------------------------
Sub EnterOnlyNumberKeys(KeyAscii As Integer)
    Select Case KeyAscii 'Nur 0 oder 9 einlesen
        Case 48 To 57, 8, 45
        Case Else
        KeyAscii = 0
    End Select
End Sub
'//------------------------------------------------------------------------
'// Function: EnterOnlyHexKeys
'//------------------------------------------------------------------------
'// Description:
'// This function used for text boxes to allow only to enter
'// hexadecimal numbers (0..9; a..f and A..F).
'//------------------------------------------------------------------------
'// Parameters:
'// IN KeyAscii:
'//------------------------------------------------------------------------
Sub EnterOnlyHexKeys(KeyAscii As Integer)
  Select Case KeyAscii 'Nur Hexziffern einlesen
    Case 48 To 57, 65 To 70, 97 To 102, 8, 120
    Case Else
      KeyAscii = 0
  End Select
  Select Case KeyAscii 'Kleinbuchstaben in Grobuchstaben
    Case 97 To 103
      KeyAscii = KeyAscii - 32
  End Select
End Sub
'//------------------------------------------------------------------------
'// Function: Hex2Dec
'//------------------------------------------------------------------------
'// Description:
'// This function converts a hexadecimal string into a decimal value.
'//------------------------------------------------------------------------
'// Parameters:
'// IN HEX: Hexadecimal string
'// Return: Decimal value
'//------------------------------------------------------------------------
Function Hex2Dec(HEX As String) As Integer
  Dim a1, a0 As String
  Dim i1, i0 As Integer
  a0 = 0
  a1 = 1
  
  If Len(HEX) >= 1 Then
     If Len(HEX) < 2 Then
       a0 = Left$(HEX, 1)
       a1 = "0"
     Else
       a1 = Left$(HEX, 1)
       a0 = Mid$(HEX, 2, 1)
     End If

     If Asc(a1) < 65 Then ' A-F ?
       i1 = 1 * (Asc(a1) - 48) 'convert to int
     Else
       i1 = (1 * (Asc(a1) - 65)) + 10 'convert to int
     End If

     If Asc(a0) < 65 Then 'A-F ?
       i0 = 1 * (Asc(a0) - 48) 'convert to int
     Else
       i0 = (1 * (Asc(a0) - 65)) + 10 'convert to int
     End If
  End If
  Hex2Dec = i0 + (i1 * 16)
End Function
'//------------------------------------------------------------------------
'// Function: BytArray2HEXString
'//------------------------------------------------------------------------
'// Description:
'// This function converts a byte array into a hexadecimal formated string.
'//------------------------------------------------------------------------
'// Parameters:
'// IN InData: Byte Array
'// IN InLen:  Byte Array Length
'// IN bSpace: False..no spaces; True..spaces between each hex. value
'// Return: Formated String
'//------------------------------------------------------------------------
Public Function BytArray2HEXString(ByRef InData() As Byte, InLen As Long, bSpace As Boolean) As String
    Dim i As Integer
    Dim bytTmpChr As Byte
    Dim strTmpString As String
    strTmpString = ""
    i = 0
    While i < InLen
        bytTmpChr = InData(i)
        If (bytTmpChr < 16) Then
            strTmpString = strTmpString + "0"
        End If
        strTmpString = strTmpString + HEX(bytTmpChr)
        If bSpace Then
            strTmpString = strTmpString + " "
        End If
        i = i + 1
    Wend
   BytArray2HEXString = strTmpString
End Function
'//------------------------------------------------------------------------
'// Function: BytArray2ASCIIString
'//------------------------------------------------------------------------
'// Description:
'// This function converts a byte array into a ASCII string.
'//------------------------------------------------------------------------
'// Parameters:
'// IN InData: Byte Array
'// IN InLen:  Byte Array Length
'// IN bSpace: False..no spaces; True..spaces between each value
'// Return: Formated String
'//------------------------------------------------------------------------
Public Function BytArray2ASCIIString(ByRef InData() As Byte, InLen As Long, bSpace As Boolean) As String
    Dim i As Integer
    Dim bytTmpChr As Byte
    Dim strTmpString As String
    strTmpString = ""
    i = 0
    While i < InLen
        bytTmpChr = InData(i)
        If (bytTmpChr < 32) Then
            strTmpString = strTmpString + "."
        Else
            strTmpString = strTmpString + Chr$(bytTmpChr)
        End If
        
        If bSpace Then
            strTmpString = strTmpString + " "
        End If
        i = i + 1
    Wend
   BytArray2ASCIIString = strTmpString
End Function



